+++
date = "2018-07-16"
weight = 100

title = "How to Build With Jenkins"

aliases = [
    "/old-wiki/Guidelines/HowToBuildWithJenkins"
]
+++

# What is Jenkins

**Jenkins** is our automation overlord, and the [Apertis Jenkins
instance](https://jenkins.apertis.org/) is where all the magic happens.

All the steps needed to build images, autoclose tasks, or deploy
documentation are orchestrated by Jenkins jobs and this document explain
how to create one, taking the
[apertis-docs](https://git.apertis.org/cgit/apertis-docs.git/tree/?id=5dd6635cd1)
repository as a reference.

Remember that these notes are inevitably bound to become obsolete, so
always check the [linked
repository](https://git.apertis.org/cgit/apertis-docs.git/) for the
latest and greatest.

# The Jenkinsfile

The
[Jenkinsfile](https://git.apertis.org/cgit/apertis-docs.git/tree/Jenkinsfile?id=5dd6635cd1)
contains all the steps that compose a
[`pipeline`](https://jenkins.io/doc/book/pipeline/syntax/):

<code>

    pipeline {
      agent {
        dockerfile {
          label 'docker-slave'
          dir 'documentation-builder'
        }
      }
      stages {
        stage("Build") {
          steps {
            sh "make"
            sh "DESTDIR=out make install"
          }
        }
        stage ("Upload") {
          when { branch 'master' }
          environment {
            NSS_WRAPPER_PASSWD = '/tmp/passwd'
            NSS_WRAPPER_GROUP = '/dev/null'
          }
          steps {
            script {
              sshagent (credentials: [ "developer.apertis.org", ] ) {
                sh 'echo docker:x:$(id -u):$(id -g):docker gecos:/tmp:/bin/false > ${NSS_WRAPPER_PASSWD}'
                sh 'LD_PRELOAD=libnss_wrapper.so rsync -e "ssh -oStrictHostKeyChecking=no" -va --no-group --no-owner --no-perms --delete-after  out/usr/share/doc/apertis-docs/apertis-docs/ jenkins@developer.apertis.org:/srv/developer.apertis.org/www'
              }
            }
          }
        }
      }
    }

</code>

## The build environment

The [`agent`](https://jenkins.io/doc/book/pipeline/syntax/#agent)
section defines where the build happens: in particular we configure the
build to happen in a Docker container defined by the
[`Dockerfile`](https://git.apertis.org/cgit/apertis-docs.git/tree/documentation-builder/Dockerfile?id=5dd6635cd1)
contained in the
[`documentation-builder`](https://git.apertis.org/cgit/apertis-docs.git/tree/documentation-builder/?id=5dd6635cd1)
directory, labelling the container `docker-slave`.

<code>

```
  agent {
    dockerfile {
      label 'docker-slave'
      dir 'documentation-builder'
    }
  }
```

</code>

### The Dockerfile

The
[`Dockerfile`](https://git.apertis.org/cgit/apertis-docs.git/tree/documentation-builder/Dockerfile?id=5dd6635cd1)
defines how the container where the actual build will happen is to be
constructed.

For this job, the container is based on the `stretch-slim` base
container available on the [upstream Docker
registry](https://store.docker.com/images/debian). We plan to set up a
Docker registry for Apertis images.

Other than installing a few packages telling `apt` to avoid interactive
questions, the `Dockerfile` uses `pip` to install the `hotdoc` python
package from PyPI.

<code>

    FROM debian:stretch-slim
    ARG DEBIAN_FRONTEND=noninteractive

    # Always install procps in case the docker file gets used in jenkins
    RUN apt update && apt-get install  --no-install-recommends -y procps

    # Bits needed to upload the documentation
    RUN apt-get update  && \
        apt-get install --no-install-recommends -y rsync \
                                                   libnss-wrapper \
                                                   openssh-client

    # Bits needed to run the build
    RUN apt-get update  && \
        apt-get install --no-install-recommends -y python3-pip \
                                                   python3-setuptools \
                                                   python3-wheel \
                                                   build-essential \
                                                   python3-dev \
                                                   libxml2-dev \
                                                   libxslt1-dev \
                                                   cmake \
                                                   libyaml-dev

    RUN pip3 install hotdoc

</code>

## The build stage

The pipeline is split in stages and each stage is composed by steps.
When [showing the status of a
job](https://jenkins.apertis.org/job/apertis-docs-upload/job/master/),
Jenkins will display the status of each stage in the pipeline, and will
use the time each stage took in previous builds to estimate how much
time is needed to complete each of them.

Stages are thus a simple way to group all the commands (steps) that
compose a single semantic action.

Our build stage is very simple, consisting of two trivial shell command
invocations:

<code>

```
    stage("Build") {
      steps {
        sh "make"
        sh "DESTDIR=out make install"
      }
    }
```

</code>

## The upload stage

The upload stage is rather more involved.

As a first step, `when { branch 'master' }` will cause the stage to be
run only when building the `master` branch of the repository. This is
because by default Jenkins automatically monitors and builds all the
branches so you can easily experiment without pushing changes to the
`master` branch.

The `environment` section defines a couple of environment variables. In
this specific case the `NSS_WRAPPER_*` variables are needed to
work-around the fact that `ssh` complains if the current user is not in
the Unix database, which is not the case in the ephemeral Docker
container, as Jenkins launches commands inside it with the credentials
of the user on the actual host outside of the container. We hope to get
rid of this noise in the future.

The first thing the actual build step specifies is that it wants to run
under a `sshagent` using the `developer.apertis.org` [credentials that
have been pre-configured in
Jenkins](https://jenkins.apertis.org/credentials/).

The first command is part of the previously described workaround: it
creates a fake Unix users database with entries for the current userid
to avoid complaints from `ssh`.

The next command is the actual deployment: ignoring the
`libnss_wrapper.so` workaround, it just invokes \`rsync\` to publish the
build artifacts.

<code>

```
    stage ("Upload") {
      when { branch 'master' }
      environment {
        NSS_WRAPPER_PASSWD = '/tmp/passwd'
        NSS_WRAPPER_GROUP = '/dev/null'
      }
      steps {
        script {
          sshagent (credentials: [ "developer.apertis.org", ] ) {
            sh 'echo docker:x:$(id -u):$(id -g):docker gecos:/tmp:/bin/false > ${NSS_WRAPPER_PASSWD}'
            sh 'LD_PRELOAD=libnss_wrapper.so rsync -e "ssh -oStrictHostKeyChecking=no" -va --no-group --no-owner --no-perms --delete-after  out/usr/share/doc/apertis-docs/apertis-docs/ jenkins@developer.apertis.org:/srv/developer.apertis.org/www'
          }
        }
      }
    }
```

</code>

# Pointing Jenkins to the `Jenkinsfile`

Once the `Jenkinsfile` has been defined, committed, and pushed, you need
to create a new corresponding job on the Jenkins instance. All the
Apertis Jenkins jobs are managed as Jenkins Job Builder templates in the
[`apertis-jenkins-jobs`](https://gitlab.apertis.org/infrastructure/apertis-jenkins-jobs)
repository.

The
[Guidelines/HowToCreateJenkinsJobs]( {{< ref "/guidelines/howtocreatejenkinsjobs.md" >}} )
page will help you to create and deploy a new pipeline job pointing to
your new `Jenkinsfile`.
