+++
date = "2019-10-24"
weight = 100

title = "How to Create Jenkins Jobs"

aliases = [
    "/old-wiki/Guidelines/HowToCreateJenkinsJobs"
]
+++

# What is Jenkins

**Jenkins** is our automation overlord, and the [Apertis Jenkins
instance](https://jenkins.apertis.org/) is where all the magic happens.

Modern Jenkins jobs are called "pipeline" jobs and usually consist of
two pieces:

  - the job itself
  - the `Jenkinsfile` which defines the actual command pipeline to be
    run

Usually each component carries its `Jenkinsfile` in their main git
repository alongside the code, for example as the [`apertis-designs`
repository](https://git.apertis.org/cgit/apertis-designs.git/tree/Jenkinsfile)
does. See the [HowToBuildWithJenkins
guide]( {{< ref "/guidelines/howtobuildwithjenkins.md" >}} ) for more
information about writing `Jenkinsfiles`.

The jobs themselves usually carry little information, just the bare
minimum to inform Jenkins where the `Jenkinsfile` is located and how to
check it out.

# Jenkins Job Builder

While Jenkins is usually controlled via its Web UI, Apertis uses
[Jenkins Job
Builder](https://docs.openstack.org/infra/jenkins-job-builder/) to
generate jobs from plain text files in order to:

  - use Git to store the job definitions
  - use the same review workflow as for code
  - easily regenerate the whole setup from scratch
  - generate multiple jobs from a single template
  - allow for changes spanning multiple jobs to be applied easily

## Job definitions

Jenkins Job Builder expects jobs to be defined with a [configuration
format based on
YAML](https://docs.openstack.org/infra/jenkins-job-builder/definition.html).

We have a bunch of [Jenkins job
templates](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/) that
handle [building
images](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/tree/image-recipes),
[publishing the
documentation](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/tree/hotdoc),
or [fetching code from git and pushing it to
OBS](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/tree/packaging).

This is an example definition taken from [the
`hotdoc/apertis-designs.yaml`
template](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/tree/hotdoc/apertis-designs.yaml):
<code>

    - job-template:
        name: '{project}-{distro}/hotdoc/apertis-designs'
        project-type: pipeline
        pipeline-scm:
          scm:
            - git:
                url: '{repobase}/apertis-designs.git'
                branches:
                  - '*/{branch}'

    - project:
        name: '{project}-{distro}/hotdoc/apertis-designs'
        jobs:
          - '{project}-{distro}/hotdoc/apertis-designs'

    - defaults:
        !include: ../defaults.yaml.inc

</code>

This definition uses `job-template` in order to make use of the
variables defined in
[`defaults.yaml.inc`](https://git.apertis.org/cgit/apertis-jenkins-jobs.git/tree/defaults.yaml.inc):

<code>

    name: global
    distro: '18.06'
    branch: 'master'
    project: 'apertis'
    maintainers: 'maintainers@lists.apertis.org'
    repobase: 'https://git.apertis.org/git'

</code>

The definition will create a single
`apertis-18.06/hotdoc/apertis-designs` pipeline job, pointing to the
<https://git.apertis.org/git/apertis-designs.git> and using the
[`Jenkinsfile`](https://git.apertis.org/cgit/apertis-designs.git/tree/Jenkinsfile)
hosted there.

## Checking the job syntax

Before sending the job definitions to the Jenkins instance, you can test
their syntax locally using `jenkins-jobs test`: <code>

    $ jenkins-jobs test hotdoc/apertis-designs.yaml | grep ^INFO
    INFO:jenkins_jobs.cli.subcommand.update:Updating jobs in ['hotdoc/apertis-designs.yaml'] ([])
    INFO:jenkins_jobs.builder:Number of jobs generated:  1
    INFO:jenkins_jobs.builder:Job name:  apertis-18.06/hotdoc/apertis-designs
    INFO:jenkins_jobs.builder:Number of views generated:  0

</code>

By omitting the `grep ^INFO` you can inspect the generated XML that will
be submitted to Jenkins.

## Uploading to playground

Rather than uploading directly to where the main jobs are hosted,
testing them in your playground is a good idea.

To do so

  - locally edit (without committing the changes) `defaults.yaml.inc` to
    point to your playground:
      - replace `project: 'apertis'` with `project:
        'playground/$USER/apertis'` (for instance, `project:
        'playground/em/apertis'`)
      - `branch: 'master'` to the branch you're working on
  - check the syntax with `jenkins-jobs test`: <code>
        $ jenkins-jobs test hotdoc/apertis-designs.yaml | grep ^INFO
        INFO:jenkins_jobs.cli.subcommand.update:Updating jobs in ['hotdoc/apertis-designs.yaml'] ([])
        INFO:jenkins_jobs.builder:Number of jobs generated:  1
        INFO:jenkins_jobs.builder:Job name:  playground/em/apertis-18.06/hotdoc/apertis-designs
        INFO:jenkins_jobs.builder:Number of views generated:  0
    </code>
  - ensure that the destination folder hierarchy has already been
    created on the Jenkins instance otherwise create it from the web UI
  - upload with `jenkins-jobs update`: <code>
        $ jenkins-jobs update hotdoc/apertis-designs.yaml
        INFO:jenkins_jobs.cli.subcommand.update:Updating jobs in ['hotdoc/apertis-designs.yaml'] ([])
        INFO:jenkins_jobs.builder:Number of jobs generated:  1
        INFO:jenkins_jobs.builder:Creating jenkins job playground/em/apertis-18.06/hotdoc/apertis-designs
        INFO:jenkins_jobs.cli.subcommand.update:Number of jobs updated: 1
        INFO:jenkins_jobs.builder:Number of views generated:  0
        INFO:jenkins_jobs.cli.subcommand.update:Number of views updated: 0
    </code>
  - check the state of the [newly created playground
    job](https://jenkins.apertis.org/job/playground/) and trigger it
    manually if needed
  - to quickly experiment, once you uploaded the job you can edit it
    directly from the Jenkins web UI, from the **Configure** pane (but
    only do that in the playground)
  - repeat until you're sure it works as you expect

## Uploading to the final location

Once you're satisfied about how the job is working in your playground,
you can now submit it for review and then promote it to its final
location:

  - submit your commits using the usual [contribution
    process](https://designs.apertis.org/latest/contribution-process.html)
  - get your commits reviewed and landed
  - revert the local edits of `defaults.yaml.inc`
  - make sure you don't have any uncommitted changes
  - upload with `jenkins-jobs update`: <code>
        $ jenkins-jobs update hotdoc/apertis-designs.yaml
        INFO:jenkins_jobs.cli.subcommand.update:Updating jobs in ['hotdoc/apertis-designs.yaml'] ([])
        INFO:jenkins_jobs.builder:Number of jobs generated:  1
        INFO:jenkins_jobs.builder:Creating jenkins job apertis-18.06/hotdoc/apertis-designs
        INFO:jenkins_jobs.cli.subcommand.update:Number of jobs updated: 1
        INFO:jenkins_jobs.builder:Number of views generated:  0
        INFO:jenkins_jobs.cli.subcommand.update:Number of views updated: 0
    </code>
  - check from the Jenkins Web UI that everything is sane
